From c79b8770444501bf4e79f0623e77b5b47e547f7c Mon Sep 17 00:00:00 2001
From: Sasha Koruga <skoruga@gmail.com>
Date: Sat, 14 Aug 2010 03:42:48 -0700
Subject: [PATCH] Added Direct3D shader (HLSL) support and GUI

---
 modules/gui/qt4/menus.cpp           |    4 +
 modules/video_output/msw/common.h   |    3 +
 modules/video_output/msw/direct3d.c |  583 +++++++++++++++++++++++++++++------
 src/video_output/vout_wrapper.c     |   42 +++
 4 files changed, 534 insertions(+), 98 deletions(-)

diff --git a/modules/gui/qt4/menus.cpp b/modules/gui/qt4/menus.cpp
index 3e682e0..2e173f1 100644
--- a/modules/gui/qt4/menus.cpp
+++ b/modules/gui/qt4/menus.cpp
@@ -227,6 +227,7 @@ static int VideoAutoMenuBuilder( vout_thread_t *p_object,
     PUSH_VAR( "video-wallpaper" );
 #ifdef WIN32
     PUSH_VAR( "direct3d-desktop" );
+	PUSH_VAR( "shader-effects" );
 #endif
     PUSH_VAR( "video-snapshot" );
     PUSH_VAR( "zoom" );
@@ -638,6 +639,9 @@ QMenu *QVLCMenu::VideoMenu( intf_thread_t *p_intf, QMenu *current )
         ACT_ADDMENU( current, "deinterlace", qtr( "&Deinterlace" ) );
         ACT_ADDMENU( current, "deinterlace-mode", qtr( "&Deinterlace mode" ) );
         ACT_ADDMENU( current, "postprocess", qtr( "&Post processing" ) );
+#ifdef WIN32  //until opengl shaders are added
+        ACT_ADDMENU( current, "shader-effects", qtr( "S&hader Effects" ) );
+#endif
     }
 
     p_input = THEMIM->getInput();
diff --git a/modules/video_output/msw/common.h b/modules/video_output/msw/common.h
index 3a67c7d..a3d495e 100644
--- a/modules/video_output/msw/common.h
+++ b/modules/video_output/msw/common.h
@@ -168,6 +168,7 @@ struct vout_display_sys_t
     bool allow_hw_yuv;    /* Should we use hardware YUV->RGB conversions */
     /* show video on desktop window ? */
     bool use_desktop;
+    char * shaderFXOption;
     struct {
         bool is_fullscreen;
         bool is_on_top;
@@ -177,6 +178,8 @@ struct vout_display_sys_t
 
     // core objects
     HINSTANCE               hd3d9_dll;       /* handle of the opened d3d9 dll */
+    HINSTANCE               hd3d9x_dll;      /* handle of the opened d3d9x dll */
+    void*                   d3dxShader;
     LPDIRECT3D9             d3dobj;
     LPDIRECT3DDEVICE9       d3ddev;
     D3DPRESENT_PARAMETERS   d3dpp;
diff --git a/modules/video_output/msw/direct3d.c b/modules/video_output/msw/direct3d.c
index 4fd89e0..8ffa927 100644
--- a/modules/video_output/msw/direct3d.c
+++ b/modules/video_output/msw/direct3d.c
@@ -4,7 +4,8 @@
  * Copyright (C) 2006-2009 the VideoLAN team
  *$Id$
  *
- * Authors: Damien Fouilleul <damienf@videolan.org>
+ * Authors: Damien Fouilleul <damienf@videolan.org>, 
+ *          Sasha Koruga <skoruga@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,9 +45,14 @@
 
 #include <windows.h>
 #include <d3d9.h>
+#include <d3dx9.h>
 
 #include "common.h"
 
+#ifndef D3DXFX_NOT_CLONEABLE
+#define D3DXFX_NOT_CLONEABLE    (1 << 11)
+#endif
+
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -57,7 +63,8 @@ static void Close(vlc_object_t *);
 #define DESKTOP_TEXT N_("Enable desktop mode ")
 #define DESKTOP_LONGTEXT N_(\
     "The desktop mode allows you to display the video on the desktop.")
-
+#define SHADERFX_MODE_TEXT N_("Shader Effects")
+#define SHADERFX_MODE_LONGTEXT N_( "Shader effect to use for video post-processing.")
 #define D3D_HELP N_("Recommended video output for Windows Vista and later versions")
 
 vlc_module_begin ()
@@ -68,6 +75,7 @@ vlc_module_begin ()
     set_subcategory(SUBCAT_VIDEO_VOUT)
 
     add_bool("direct3d-desktop", false, NULL, DESKTOP_TEXT, DESKTOP_LONGTEXT, true)
+    add_string( "shader-effects", "Disabled", NULL, SHADERFX_MODE_TEXT, SHADERFX_MODE_LONGTEXT, true )
 
     set_capability("vout display", 150)
     add_shortcut("direct3d")
@@ -82,6 +90,172 @@ vlc_module_begin ()
 vlc_module_end ()
 
 /*****************************************************************************
+ * HLSL Pixel Shader Code
+ *****************************************************************************/
+char shaderCode[] = " \
+sampler2D screen; \
+float4 rgb_to_yuv601(float4 RGB) \
+{ \
+    float Kr = 0.299; \
+    float Kg = 0.587; \
+    float Kb = 0.114; \
+    float Y = Kr*RGB.r + Kg*RGB.g + Kb*RGB.b; \
+    float V = (RGB.r-Y)/(1-Kr); \
+    float U = (RGB.b-Y)/(1-Kb); \
+    return float4(Y,U,V,1); \
+} \
+float4 yuv709_to_rgb(float4 YUV) \
+{ \
+    float Kr = 0.2125; \
+    float Kg = 0.7154; \
+    float Kb = 0.0721; \
+    float Y = YUV.x; \
+    float U = YUV.y; \
+    float V = YUV.z; \
+    float R = Y + V*(1-Kr); \
+    float G = Y - U*(1-Kb)*Kb/Kg - V*(1-Kr)*Kr/Kg; \
+    float B = Y + U*(1-Kb); \
+    return float4(R,G,B,1); \
+} \
+float4 Invert(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    color.r = 1.0 - color.r; \
+    color.g = 1.0 - color.g; \
+    color.b = 1.0 - color.b; \
+    return color; \
+} \
+float4 Convert601to709(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    return yuv709_to_rgb(rgb_to_yuv601(color)); \
+} \
+float4 GammaCorrection18(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    color = pow(color,1.0/1.8); \
+    return color; \
+} \
+float4 GammaCorrection22(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    color = pow(color,1.0/2.2); \
+    return color; \
+} \
+float4 GammaCorrectionBT709(float2 screenCoords : TEXCOORD0) : COLOR0  \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    if(color.r > 0.018) \
+        color.r = 1.099 * pow(color.r,0.45) - 0.099; \
+    else \
+        color.r = 4.5138 * color.r; \
+    if(color.g > 0.018) \
+        color.g = 1.099 * pow(color.g,0.45) - 0.099; \
+    else \
+        color.g = 4.5138 * color.g; \
+    if(color.b > 0.018) \
+        color.b = 1.099 * pow(color.b,0.45) - 0.099; \
+    else \
+        color.b = 4.5138 * color.b; \
+    return color; \
+} \
+float4 WidenColorSpace(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    color.r = max(color.r - 0.0627450980392157,0) * 1.164383561643836; \
+    color.g = max(color.g - 0.0627450980392157,0) * 1.164383561643836; \
+    color.b = max(color.b - 0.0627450980392157,0) * 1.164383561643836; \
+    return saturate(color); \
+} \
+float4 Grayscale(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    float gray = 0.2989 * color.r + 0.5870 * color.g + 0.1140 * color.b; \
+    color.r = color.g = color.b = gray; \
+    return saturate(color); \
+} \
+float4 Detect15(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    float4 color = tex2D( screen, screenCoords.xy); \
+    for(int i = 0; i < 3; ++i) \
+    { \
+            if(color[i] < .03 || color[i] >.97) \
+            { \
+                color[0] = 1; color[1] = color[2] = 0; \
+            } \
+    } \
+    return saturate(color); \
+} \
+float4 disabled(float2 screenCoords : TEXCOORD0) : COLOR0 \
+{ \
+    return saturate(tex2D( screen, screenCoords.xy)); \
+} \
+technique invert \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 Invert(); \
+    } \
+} \
+technique convert601to709 \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 Convert601to709(); \
+    } \
+} \
+technique gammacorrection18 \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 GammaCorrection18(); \
+    } \
+} \
+technique gammacorrection22 \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 GammaCorrection22(); \
+    } \
+} \
+technique gammacorrectionbt709 \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 GammaCorrectionBT709(); \
+    } \
+} \
+technique detect15 \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 Detect15(); \
+    } \
+} \
+technique widencolorspace \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 WidenColorSpace(); \
+    } \
+} \
+technique grayscale \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 Grayscale(); \
+    } \
+} \
+technique Disabled \
+{ \
+    pass p1 \
+    { \
+        PixelShader = compile ps_2_0 disabled(); \
+    } \
+} \
+    ";
+
+/*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
 struct picture_sys_t
@@ -109,6 +283,7 @@ static void Direct3DRenderScene(vout_display_t *vd, LPDIRECT3DSURFACE9 surface);
 
 /* */
 static int DesktopCallback(vlc_object_t *, char const *, vlc_value_t, vlc_value_t, void *);
+static int FXCallback(vlc_object_t *, char const *, vlc_value_t, vlc_value_t, void *);
 
 /**
  * It creates a Direct3D vout display.
@@ -123,7 +298,8 @@ static int Open(vlc_object_t *object)
     if (!sys)
         return VLC_ENOMEM;
 
-    if (Direct3DCreate(vd)) {
+    if (Direct3DCreate(vd))
+    {
         msg_Err(vd, "Direct3D could not be initialized");
         Direct3DDestroy(vd);
         free(sys);
@@ -131,6 +307,7 @@ static int Open(vlc_object_t *object)
     }
 
     sys->use_desktop = var_CreateGetBool(vd, "direct3d-desktop");
+    sys->shaderFXOption = var_CreateGetString(vd, "shader-effects");
     sys->reset_device = false;
     sys->reset_device = false;
     sys->allow_hw_yuv = var_CreateGetBool(vd, "directx-hw-yuv");
@@ -146,7 +323,8 @@ static int Open(vlc_object_t *object)
 
     /* */
     video_format_t fmt;
-    if (Direct3DOpen(vd, &fmt)) {
+    if (Direct3DOpen(vd, &fmt))
+    {
         msg_Err(vd, "Direct3D could not be opened");
         goto error;
     }
@@ -168,6 +346,8 @@ static int Open(vlc_object_t *object)
     val.psz_string = _("Desktop");
     var_Change(vd, "direct3d-desktop", VLC_VAR_SETTEXT, &val, NULL);
     var_AddCallback(vd, "direct3d-desktop", DesktopCallback, NULL);
+    var_Change(vd, "shader-effects", VLC_VAR_SETTEXT, &val, NULL);
+    var_AddCallback(vd, "shader-effects", FXCallback, NULL);
 
     /* Setup vout_display now that everything is fine */
     vd->fmt  = fmt;
@@ -219,6 +399,7 @@ static void Close(vlc_object_t *object)
     vout_display_t * vd = (vout_display_t *)object;
 
     var_DelCallback(vd, "direct3d-desktop", DesktopCallback, NULL);
+    var_DelCallback(vd, "shader-effects", FXCallback, NULL);
 
     Direct3DClose(vd);
 
@@ -268,7 +449,8 @@ static void Display(vout_display_t *vd, picture_t *picture)
     const RECT src = sys->rect_dest_clipped;
     const RECT dst = sys->rect_dest_clipped;
     HRESULT hr = IDirect3DDevice9_Present(d3ddev, &src, &dst, NULL, NULL);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
     }
 
@@ -290,7 +472,8 @@ static int ControlReopenDevice(vout_display_t *vd)
 {
     vout_display_sys_t *sys = vd->sys;
 
-    if (!sys->use_desktop) {
+    if (!sys->use_desktop)
+    {
         /* Save non-desktop state */
         sys->desktop_save.is_fullscreen = vd->cfg->is_fullscreen;
         sys->desktop_save.is_on_top     = sys->is_on_top;
@@ -314,7 +497,8 @@ static int ControlReopenDevice(vout_display_t *vd)
     event_cfg_t cfg;
     memset(&cfg, 0, sizeof(cfg));
     cfg.use_desktop = sys->use_desktop;
-    if (!sys->use_desktop) {
+    if (!sys->use_desktop)
+    {
         cfg.win.type   = VOUT_WINDOW_TYPE_HWND;
         cfg.win.x      = sys->desktop_save.win.left;
         cfg.win.y      = sys->desktop_save.win.top;
@@ -323,7 +507,8 @@ static int ControlReopenDevice(vout_display_t *vd)
     }
 
     event_hwnd_t hwnd;
-    if (EventThreadStart(sys->event, &hwnd, &cfg)) {
+    if (EventThreadStart(sys->event, &hwnd, &cfg))
+    {
         msg_Err(vd, "Failed to restart event thread");
         return VLC_EGENERIC;
     }
@@ -336,7 +521,8 @@ static int ControlReopenDevice(vout_display_t *vd)
 
     /* */
     video_format_t fmt;
-    if (Direct3DOpen(vd, &fmt)) {
+    if (Direct3DOpen(vd, &fmt))
+    {
         CommonClean(vd);
         msg_Err(vd, "Failed to reopen device");
         return VLC_EGENERIC;
@@ -344,7 +530,8 @@ static int ControlReopenDevice(vout_display_t *vd)
     vd->fmt = fmt;
     sys->is_first_display = true;
 
-    if (sys->use_desktop) {
+    if (sys->use_desktop)
+    {
         /* Disable fullscreen/on_top while using desktop */
         if (sys->desktop_save.is_fullscreen)
             vout_display_SendEventFullscreen(vd, false);
@@ -363,17 +550,23 @@ static int Control(vout_display_t *vd, int query, va_list args)
 {
     vout_display_sys_t *sys = vd->sys;
 
-    switch (query) {
+    switch (query)
+    {
     case VOUT_DISPLAY_RESET_PICTURES:
         /* FIXME what to do here in case of failure */
-        if (sys->reset_device) {
-            if (ControlResetDevice(vd)) {
+        if (sys->reset_device)
+        {
+            if (ControlResetDevice(vd))
+            {
                 msg_Err(vd, "Failed to reset device");
                 return VLC_EGENERIC;
             }
             sys->reset_device = false;
-        } else if(sys->reopen_device) {
-            if (ControlReopenDevice(vd)) {
+        }
+        else if(sys->reopen_device)
+        {
+            if (ControlReopenDevice(vd))
+            {
                 msg_Err(vd, "Failed to reopen device");
                 return VLC_EGENERIC;
             }
@@ -396,7 +589,8 @@ static void Manage (vout_display_t *vd)
     sys->ch_desktop = false;
     vlc_mutex_unlock(&sys->lock);
 
-    if (ch_desktop) {
+    if (ch_desktop)
+    {
         sys->reopen_device = true;
         vout_display_SendEventPicturesInvalid(vd);
     }
@@ -405,7 +599,8 @@ static void Manage (vout_display_t *vd)
     /*
      * Position Change
      */
-    if (sys->changes & DX_POSITION_CHANGE) {
+    if (sys->changes & DX_POSITION_CHANGE)
+    {
 #if 0 /* need that when bicubic filter is available */
         RECT rect;
         UINT width, height;
@@ -435,7 +630,8 @@ static int Direct3DCreate(vout_display_t *vd)
     vout_display_sys_t *sys = vd->sys;
 
     sys->hd3d9_dll = LoadLibrary(TEXT("D3D9.DLL"));
-    if (!sys->hd3d9_dll) {
+    if (!sys->hd3d9_dll)
+    {
         msg_Warn(vd, "cannot load d3d9.dll, aborting");
         return VLC_EGENERIC;
     }
@@ -443,26 +639,50 @@ static int Direct3DCreate(vout_display_t *vd)
     LPDIRECT3D9 (WINAPI *OurDirect3DCreate9)(UINT SDKVersion);
     OurDirect3DCreate9 =
         (void *)GetProcAddress(sys->hd3d9_dll, TEXT("Direct3DCreate9"));
-    if (!OurDirect3DCreate9) {
+    if (!OurDirect3DCreate9)
+    {
         msg_Err(vd, "Cannot locate reference to Direct3DCreate9 ABI in DLL");
         return VLC_EGENERIC;
     }
 
     /* Create the D3D object. */
     LPDIRECT3D9 d3dobj = OurDirect3DCreate9(D3D_SDK_VERSION);
-    if (!d3dobj) {
+    if (!d3dobj)
+    {
        msg_Err(vd, "Could not create Direct3D9 instance.");
        return VLC_EGENERIC;
     }
     sys->d3dobj = d3dobj;
 
+    //load shader support
+    for(int i = 43; i > 23; --i)
+    {
+        char buffer[3], filename[14];
+        strcpy(filename, "D3dx9_");
+        strcat(filename, itoa(i,buffer,10));
+        strcat(filename, ".dll");
+        sys->hd3d9x_dll = LoadLibrary(filename);
+        msg_Dbg(vd, "Attempting to load: %s", filename);
+        if(sys->hd3d9x_dll)
+        {
+            msg_Dbg(vd, "loaded: %s", filename);
+            break;
+        }
+    }
+    if(!sys->hd3d9x_dll)
+    {
+        msg_Warn(vd, "cannot load D3dx9d_[24-43].dll; HLSL pixel shading will be disabled. (Please install DirectX end-user runtime)");
+        sys->d3dxShader = NULL;
+    }
+
     /*
     ** Get device capabilities
     */
     D3DCAPS9 d3dCaps;
     ZeroMemory(&d3dCaps, sizeof(d3dCaps));
     HRESULT hr = IDirect3D9_GetDeviceCaps(d3dobj, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &d3dCaps);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
        msg_Err(vd, "Could not read adapter capabilities. (hr=0x%lX)", hr);
        return VLC_EGENERIC;
     }
@@ -479,12 +699,21 @@ static void Direct3DDestroy(vout_display_t *vd)
     vout_display_sys_t *sys = vd->sys;
 
     if (sys->d3dobj)
-       IDirect3D9_Release(sys->d3dobj);
+    {
+        IDirect3D9_Release(sys->d3dobj);
+        sys->d3dobj = NULL;
+    }
     if (sys->hd3d9_dll)
+    {
         FreeLibrary(sys->hd3d9_dll);
+        sys->hd3d9_dll = NULL;
+    }
+    if (sys->hd3d9x_dll)
+    {
+        FreeLibrary(sys->hd3d9x_dll);
+        sys->hd3d9x_dll = NULL;
+    }
 
-    sys->d3dobj = NULL;
-    sys->hd3d9_dll = NULL;
 }
 
 
@@ -503,7 +732,8 @@ static int Direct3DFillPresentationParameters(vout_display_t *vd)
     D3DDISPLAYMODE d3ddm;
     HRESULT hr = IDirect3D9_GetAdapterDisplayMode(sys->d3dobj,
                                                   D3DADAPTER_DEFAULT, &d3ddm);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
        msg_Err(vd, "Could not read adapter display mode. (hr=0x%lX)", hr);
        return VLC_EGENERIC;
     }
@@ -514,9 +744,9 @@ static int Direct3DFillPresentationParameters(vout_display_t *vd)
     d3dpp->Flags                  = D3DPRESENTFLAG_VIDEO;
     d3dpp->Windowed               = TRUE;
     d3dpp->hDeviceWindow          = vd->sys->hvideownd;
-    d3dpp->BackBufferWidth        = __MAX(GetSystemMetrics(SM_CXVIRTUALSCREEN),
+    d3dpp->BackBufferWidth        = __MAX((unsigned int)GetSystemMetrics(SM_CXVIRTUALSCREEN),
                                           d3ddm.Width);
-    d3dpp->BackBufferHeight       = __MAX(GetSystemMetrics(SM_CYVIRTUALSCREEN),
+    d3dpp->BackBufferHeight       = __MAX((unsigned int)GetSystemMetrics(SM_CYVIRTUALSCREEN),
                                           d3ddm.Height);
     d3dpp->SwapEffect             = D3DSWAPEFFECT_COPY;
     d3dpp->MultiSampleType        = D3DMULTISAMPLE_NONE;
@@ -552,12 +782,31 @@ static int Direct3DOpen(vout_display_t *vd, video_format_t *fmt)
 
     // Create the D3DDevice
     LPDIRECT3DDEVICE9 d3ddev;
-    HRESULT hr = IDirect3D9_CreateDevice(d3dobj, D3DADAPTER_DEFAULT,
-                                         D3DDEVTYPE_HAL, sys->hvideownd,
+    
+    // Look for 'NVIDIA PerfHUD' adapter
+    // If it is present, override default settings
+    UINT AdapterToUse = D3DADAPTER_DEFAULT;
+    D3DDEVTYPE DeviceType = D3DDEVTYPE_HAL;
+    for (UINT Adapter=0; Adapter< IDirect3D9_GetAdapterCount(d3dobj); ++Adapter)
+    {
+        D3DADAPTER_IDENTIFIER9 Identifier;
+        HRESULT Res;
+        Res = IDirect3D9_GetAdapterIdentifier(d3dobj,Adapter,0,&Identifier);
+        if (strstr(Identifier.Description,"PerfHUD") != 0)
+        {
+            AdapterToUse = Adapter;
+            DeviceType = D3DDEVTYPE_REF;
+            break;
+        }
+    }
+
+    HRESULT hr = IDirect3D9_CreateDevice(d3dobj, AdapterToUse,
+                                         DeviceType, sys->hvideownd,
                                          D3DCREATE_SOFTWARE_VERTEXPROCESSING|
                                          D3DCREATE_MULTITHREADED,
                                          &sys->d3dpp, &d3ddev);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
        msg_Err(vd, "Could not create the D3D device! (hr=0x%lX)", hr);
        return VLC_EGENERIC;
     }
@@ -565,11 +814,46 @@ static int Direct3DOpen(vout_display_t *vd, video_format_t *fmt)
 
     UpdateRects(vd, NULL, NULL, true);
 
-    if (Direct3DCreateResources(vd, fmt)) {
+    if (Direct3DCreateResources(vd, fmt))
+    {
         msg_Err(vd, "Failed to allocate resources");
         return VLC_EGENERIC;
     }
 
+    if(sys->hd3d9x_dll)
+    {
+        HRESULT (WINAPI * OurD3DXCreateEffect)(
+            LPDIRECT3DDEVICE9               pDevice,
+            LPCVOID                         pSrcData,
+            UINT                            SrcDataLen,
+            CONST D3DXMACRO*                pDefines,
+            LPD3DXINCLUDE                   pInclude,
+            DWORD                           Flags,
+            LPD3DXEFFECTPOOL                pPool,
+            LPD3DXEFFECT*                   ppEffect,
+            LPD3DXBUFFER*                   ppCompilationErrors);
+
+        OurD3DXCreateEffect = (void*)GetProcAddress(sys->hd3d9x_dll, TEXT("D3DXCreateEffect"));
+        if (!OurD3DXCreateEffect)
+        {
+            msg_Warn(vd, "Cannot locate reference to D3DXCreateEffect ABI in DLL; pixel shading will be disabled");
+            sys->d3dxShader = NULL;
+        }
+        else
+        {
+            LPD3DXBUFFER buffer = NULL;
+            DWORD dwShaderFlags = D3DXFX_NOT_CLONEABLE;
+            HRESULT hr = OurD3DXCreateEffect(sys->d3ddev, shaderCode, strlen(shaderCode), NULL, NULL, dwShaderFlags, NULL, ((ID3DXEffect**)(&sys->d3dxShader)), &buffer);
+            if (FAILED(hr))
+            {
+               msg_Warn(vd, "OurD3DXCreateEffect Error (hr=0x%lX) -- pixel shading will be disabled.", hr);
+               if(buffer)
+                   msg_Warn(vd, "HLSL Compilation Error: %s", (char*)buffer->lpVtbl->GetBufferPointer(buffer));
+               sys->d3dxShader = NULL;
+            }
+        }
+    }
+
     /* Change the window title bar text */
     EventThreadUpdateTitle(sys->event, VOUT_TITLE " (Direct3D output)");
 
@@ -586,10 +870,16 @@ static void Direct3DClose(vout_display_t *vd)
 
     Direct3DDestroyResources(vd);
 
+    if(sys->d3dxShader)
+    {
+        ((ID3DXEffect*)(sys->d3dxShader))->lpVtbl->Release(sys->d3dxShader);
+        sys->d3dxShader = NULL;
+    }
     if (sys->d3ddev)
+    {
        IDirect3DDevice9_Release(sys->d3ddev);
-
-    sys->d3ddev = NULL;
+        sys->d3ddev = NULL;
+    }
 }
 
 /**
@@ -608,7 +898,8 @@ static int Direct3DReset(vout_display_t *vd)
 
     /* */
     HRESULT hr = IDirect3DDevice9_Reset(d3ddev, &sys->d3dpp);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Err(vd, "%s failed ! (hr=%08lX)", __FUNCTION__, hr);
         return VLC_EGENERIC;
     }
@@ -616,7 +907,8 @@ static int Direct3DReset(vout_display_t *vd)
     UpdateRects(vd, NULL, NULL, true);
 
     /* re-create them */
-    if (Direct3DCreateResources(vd, &vd->fmt)) {
+    if (Direct3DCreateResources(vd, &vd->fmt))
+    {
         msg_Dbg(vd, "%s failed !", __FUNCTION__);
         return VLC_EGENERIC;
     }
@@ -635,11 +927,13 @@ static void Direct3DDestroyScene(vout_display_t *vd);
  */
 static int Direct3DCreateResources(vout_display_t *vd, video_format_t *fmt)
 {
-    if (Direct3DCreatePool(vd, fmt)) {
+    if (Direct3DCreatePool(vd, fmt))
+    {
         msg_Err(vd, "Direct3D picture pool initialization failed");
         return VLC_EGENERIC;
     }
-    if (Direct3DCreateScene(vd)) {
+    if (Direct3DCreateScene(vd))
+    {
         msg_Err(vd, "Direct3D scene initialization failed !");
         return VLC_EGENERIC;
     }
@@ -668,7 +962,8 @@ static int Direct3DCheckConversion(vout_display_t *vd,
     hr = IDirect3D9_CheckDeviceFormat(d3dobj, D3DADAPTER_DEFAULT,
                                       D3DDEVTYPE_HAL, dst, 0,
                                       D3DRTYPE_SURFACE, src);
-    if (SUCCEEDED(hr)) {
+    if (SUCCEEDED(hr))
+    {
         /* test whether device can perform color-conversion
         ** from that format to target format
         */
@@ -677,7 +972,8 @@ static int Direct3DCheckConversion(vout_display_t *vd,
                                                     D3DDEVTYPE_HAL,
                                                     src, dst);
     }
-    if (!SUCCEEDED(hr)) {
+    if (!SUCCEEDED(hr))
+    {
         if (D3DERR_NOTAVAILABLE != hr)
             msg_Err(vd, "Could not query adapter supported formats. (hr=0x%lX)", hr);
         return VLC_EGENERIC;
@@ -717,7 +1013,8 @@ static const d3d_format_t *Direct3DFindFormat(vout_display_t *vd, vlc_fourcc_t c
 {
     vout_display_sys_t *sys = vd->sys;
 
-    for (unsigned pass = 0; pass < 2; pass++) {
+    for (unsigned pass = 0; pass < 2; pass++)
+    {
         const vlc_fourcc_t *list;
 
         if (pass == 0 && sys->allow_hw_yuv && vlc_fourcc_IsYUV(chroma))
@@ -727,8 +1024,10 @@ static const d3d_format_t *Direct3DFindFormat(vout_display_t *vd, vlc_fourcc_t c
         else
             continue;
 
-        for (unsigned i = 0; list[i] != 0; i++) {
-            for (unsigned j = 0; d3d_formats[j].name; j++) {
+        for (unsigned i = 0; list[i] != 0; i++)
+        {
+            for (unsigned j = 0; d3d_formats[j].name; j++)
+            {
                 const d3d_format_t *format = &d3d_formats[j];
 
                 if (format->fourcc != list[i])
@@ -736,7 +1035,8 @@ static const d3d_format_t *Direct3DFindFormat(vout_display_t *vd, vlc_fourcc_t c
 
                 msg_Warn(vd, "trying surface pixel format: %s",
                          format->name);
-                if (!Direct3DCheckConversion(vd, format->format, target)) {
+                if (!Direct3DCheckConversion(vd, format->format, target))
+                {
                     msg_Dbg(vd, "selected surface pixel format is %s",
                             format->name);
                     return format;
@@ -757,7 +1057,8 @@ static int Direct3DLockSurface(picture_t *picture)
     /* Lock the surface to get a valid pointer to the picture buffer */
     D3DLOCKED_RECT d3drect;
     HRESULT hr = IDirect3DSurface9_LockRect(picture->p_sys->surface, &d3drect, NULL, 0);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         //msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return CommonUpdatePicture(picture, &picture->p_sys->fallback, NULL, 0);
     }
@@ -771,10 +1072,7 @@ static int Direct3DLockSurface(picture_t *picture)
 static void Direct3DUnlockSurface(picture_t *picture)
 {
     /* Unlock the Surface */
-    HRESULT hr = IDirect3DSurface9_UnlockRect(picture->p_sys->surface);
-    if (FAILED(hr)) {
-        //msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
-    }
+    IDirect3DSurface9_UnlockRect(picture->p_sys->surface);
 }
 
 /**
@@ -797,7 +1095,8 @@ static int Direct3DCreatePool(vout_display_t *vd, video_format_t *fmt)
      * the requested chroma which is usable by the hardware in an offscreen surface, as they
      * typically support more formats than textures */
     const d3d_format_t *d3dfmt = Direct3DFindFormat(vd, fmt->i_chroma, sys->d3dpp.BackBufferFormat);
-    if (!d3dfmt) {
+    if (!d3dfmt)
+    {
         msg_Err(vd, "surface pixel format is not supported.");
         return VLC_EGENERIC;
     }
@@ -818,7 +1117,8 @@ static int Direct3DCreatePool(vout_display_t *vd, video_format_t *fmt)
                                                               D3DPOOL_DEFAULT,
                                                               &surface,
                                                               NULL);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Err(vd, "Failed to create picture surface. (hr=0x%lx)", hr);
         return VLC_EGENERIC;
     }
@@ -828,19 +1128,22 @@ static int Direct3DCreatePool(vout_display_t *vd, video_format_t *fmt)
     /* Create the associated picture */
     picture_resource_t *rsc = &sys->resource;
     rsc->p_sys = malloc(sizeof(*rsc->p_sys));
-    if (!rsc->p_sys) {
+    if (!rsc->p_sys)
+    {
         IDirect3DSurface9_Release(surface);
         return VLC_ENOMEM;
     }
     rsc->p_sys->surface = surface;
     rsc->p_sys->fallback = NULL;
-    for (int i = 0; i < PICTURE_PLANE_MAX; i++) {
+    for (int i = 0; i < PICTURE_PLANE_MAX; i++)
+    {
         rsc->p[i].p_pixels = NULL;
         rsc->p[i].i_pitch = 0;
         rsc->p[i].i_lines = fmt->i_height / (i > 0 ? 2 : 1);
     }
     picture_t *picture = picture_NewFromResource(fmt, rsc);
-    if (!picture) {
+    if (!picture)
+    {
         IDirect3DSurface9_Release(surface);
         free(rsc->p_sys);
         return VLC_ENOMEM;
@@ -855,7 +1158,8 @@ static int Direct3DCreatePool(vout_display_t *vd, video_format_t *fmt)
     pool_cfg.unlock        = Direct3DUnlockSurface;
 
     sys->pool = picture_pool_NewExtended(&pool_cfg);
-    if (!sys->pool) {
+    if (!sys->pool)
+    {
         picture_Release(picture);
         IDirect3DSurface9_Release(surface);
         return VLC_ENOMEM;
@@ -869,7 +1173,8 @@ static void Direct3DDestroyPool(vout_display_t *vd)
 {
     vout_display_sys_t *sys = vd->sys;
 
-    if (sys->pool) {
+    if (sys->pool)
+    {
         picture_resource_t *rsc = &sys->resource;
         IDirect3DSurface9_Release(rsc->p_sys->surface);
         if (rsc->p_sys->fallback)
@@ -914,7 +1219,8 @@ static int Direct3DCreateScene(vout_display_t *vd)
                                         D3DPOOL_DEFAULT,
                                         &d3dtex,
                                         NULL);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Err(vd, "Failed to create texture. (hr=0x%lx)", hr);
         return VLC_EGENERIC;
     }
@@ -930,7 +1236,8 @@ static int Direct3DCreateScene(vout_display_t *vd)
                                              D3DPOOL_DEFAULT,
                                              &d3dvtc,
                                              NULL);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Err(vd, "Failed to create vertex buffer. (hr=0x%lx)", hr);
         IDirect3DTexture9_Release(d3dtex);
         return VLC_EGENERIC;
@@ -1016,14 +1323,17 @@ static void Direct3DDestroyScene(vout_display_t *vd)
  */
 static void Direct3DRenderScene(vout_display_t *vd, LPDIRECT3DSURFACE9 surface)
 {
+    UINT shaderPasses;
     vout_display_sys_t *sys = vd->sys;
     LPDIRECT3DDEVICE9 d3ddev = sys->d3ddev;
     HRESULT hr;
 
     // check if device is still available
     hr = IDirect3DDevice9_TestCooperativeLevel(d3ddev);
-    if (FAILED(hr)) {
-        if (hr == D3DERR_DEVICENOTRESET && !sys->reset_device) {
+    if (FAILED(hr))
+    {
+        if (hr == D3DERR_DEVICENOTRESET && !sys->reset_device)
+        {
             vout_display_SendEventPicturesInvalid(vd);
             sys->reset_device = true;
         }
@@ -1036,13 +1346,15 @@ static void Direct3DRenderScene(vout_display_t *vd, LPDIRECT3DSURFACE9 surface)
     /* Clear the backbuffer and the zbuffer */
     hr = IDirect3DDevice9_Clear(d3ddev, 0, NULL, D3DCLEAR_TARGET,
                               D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
     /*  retrieve picture surface */
     LPDIRECT3DSURFACE9 d3dsrc = surface;
-    if (!d3dsrc) {
+    if (!d3dsrc)
+    {
         msg_Dbg(vd, "no surface to render ?");
         return;
     }
@@ -1050,7 +1362,8 @@ static void Direct3DRenderScene(vout_display_t *vd, LPDIRECT3DSURFACE9 surface)
     /* retrieve texture top-level surface */
     LPDIRECT3DSURFACE9 d3ddest;
     hr = IDirect3DTexture9_GetSurfaceLevel(d3dtex, 0, &d3ddest);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
@@ -1062,15 +1375,17 @@ static void Direct3DRenderScene(vout_display_t *vd, LPDIRECT3DSURFACE9 surface)
 
     hr = IDirect3DDevice9_StretchRect(d3ddev, d3dsrc, &src, d3ddest, &dst, D3DTEXF_LINEAR);
     IDirect3DSurface9_Release(d3ddest);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
 
     /* Update the vertex buffer */
     CUSTOMVERTEX *vertices;
-    hr = IDirect3DVertexBuffer9_Lock(d3dvtc, 0, 0, &vertices, D3DLOCK_DISCARD);
-    if (FAILED(hr)) {
+    hr = IDirect3DVertexBuffer9_Lock(d3dvtc, 0, 0, (void**)&vertices, D3DLOCK_DISCARD);
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
@@ -1114,56 +1429,106 @@ static void Direct3DRenderScene(vout_display_t *vd, LPDIRECT3DSURFACE9 surface)
     vertices[3].tv      = 1.0f;
 
     hr= IDirect3DVertexBuffer9_Unlock(d3dvtc);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
 
     // Begin the scene
     hr = IDirect3DDevice9_BeginScene(d3ddev);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
 
-    // Setup our texture. Using textures introduces the texture stage states,
-    // which govern how textures get blended together (in the case of multiple
-    // textures) and lighting information. In this case, we are modulating
-    // (blending) our texture with the diffuse color of the vertices.
-    hr = IDirect3DDevice9_SetTexture(d3ddev, 0, (LPDIRECT3DBASETEXTURE9)d3dtex);
-    if (FAILED(hr)) {
-        msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
-        IDirect3DDevice9_EndScene(d3ddev);
-        return;
-    }
+    if(((ID3DXEffect*)(sys->d3dxShader)))
+    {
+        hr = ((ID3DXEffect*)(sys->d3dxShader))->lpVtbl->SetTechnique(((ID3DXEffect*)(sys->d3dxShader)), sys->shaderFXOption );
+        if (FAILED(hr))
+        {
+           msg_Err(vd, "SetTechnique failed (hr=0x%lX)", hr);
+        }
 
-    // Render the vertex buffer contents
-    hr = IDirect3DDevice9_SetStreamSource(d3ddev, 0, d3dvtc, 0, sizeof(CUSTOMVERTEX));
-    if (FAILED(hr)) {
-        msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
-        IDirect3DDevice9_EndScene(d3ddev);
-        return;
+        hr = ((ID3DXEffect*)(sys->d3dxShader))->lpVtbl->Begin(((ID3DXEffect*)(sys->d3dxShader)), &shaderPasses, 0 );
+        if (FAILED(hr))
+        {
+           msg_Err(vd, "Begin failed (hr=0x%lX)", hr);
+        }
     }
+    else
+        shaderPasses = 1;
 
-    // we use FVF instead of vertex shader
-    hr = IDirect3DDevice9_SetFVF(d3ddev, D3DFVF_CUSTOMVERTEX);
-    if (FAILED(hr)) {
-        msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
-        IDirect3DDevice9_EndScene(d3ddev);
-        return;
-    }
+    for( UINT uPass = 0; uPass < shaderPasses; ++uPass )
+    {
+        if(((ID3DXEffect*)(sys->d3dxShader)))
+        {
+            hr = ((ID3DXEffect*)(sys->d3dxShader))->lpVtbl->BeginPass(((ID3DXEffect*)(sys->d3dxShader)), uPass);
+            if (FAILED(hr))
+                msg_Err(vd, "BeginPass failed (hr=0x%lX)", hr);
+        }
+
+        // Setup our texture. Using textures introduces the texture stage states,
+        // which govern how textures get blended together (in the case of multiple
+        // textures) and lighting information. In this case, we are modulating
+        // (blending) our texture with the diffuse color of the vertices.
+        hr = IDirect3DDevice9_SetTexture(d3ddev, 0, (LPDIRECT3DBASETEXTURE9)d3dtex);
+        if (FAILED(hr))
+        {
+            msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
+            IDirect3DDevice9_EndScene(d3ddev);
+            return;
+        }
+
+        // Render the vertex buffer contents
+        hr = IDirect3DDevice9_SetStreamSource(d3ddev, 0, d3dvtc, 0, sizeof(CUSTOMVERTEX));
+        if (FAILED(hr))
+        {
+            msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
+            IDirect3DDevice9_EndScene(d3ddev);
+            return;
+        }
+
+        // we use FVF instead of vertex shader
+        hr = IDirect3DDevice9_SetFVF(d3ddev, D3DFVF_CUSTOMVERTEX);
+        if (FAILED(hr))
+        {
+            msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
+            IDirect3DDevice9_EndScene(d3ddev);
+            return;
+        }
+
+        // draw rectangle
+        hr = IDirect3DDevice9_DrawPrimitive(d3ddev, D3DPT_TRIANGLEFAN, 0, 2);
+        if (FAILED(hr))
+        {
+            msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
+            IDirect3DDevice9_EndScene(d3ddev);
+            return;
+        }
+
+
+        if(((ID3DXEffect*)(sys->d3dxShader)))
+        {
+            hr = ((ID3DXEffect*)(sys->d3dxShader))->lpVtbl->EndPass(((ID3DXEffect*)(sys->d3dxShader)));
+            if (FAILED(hr))
+                msg_Err(vd, "EndPass failed (hr=0x%lX)", hr);
+        }
 
-    // draw rectangle
-    hr = IDirect3DDevice9_DrawPrimitive(d3ddev, D3DPT_TRIANGLEFAN, 0, 2);
-    if (FAILED(hr)) {
-        msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
-        IDirect3DDevice9_EndScene(d3ddev);
-        return;
     }
 
+    if(((ID3DXEffect*)(sys->d3dxShader)))
+    {
+        hr = ((ID3DXEffect*)(sys->d3dxShader))->lpVtbl->End(((ID3DXEffect*)(sys->d3dxShader)));
+        if (FAILED(hr))
+           msg_Err(vd, "End failed (hr=0x%lX)", hr);
+    }
+    
     // End the scene
     hr = IDirect3DDevice9_EndScene(d3ddev);
-    if (FAILED(hr)) {
+    if (FAILED(hr))
+    {
         msg_Dbg(vd, "%s:%d (hr=0x%0lX)", __FUNCTION__, __LINE__, hr);
         return;
     }
@@ -1189,7 +1554,8 @@ static int DesktopCallback(vlc_object_t *object, char const *psz_cmd,
     vlc_mutex_unlock(&sys->lock);
 
     /* FIXME we should have a way to export variable to be saved */
-    if (ch_desktop) {
+    if (ch_desktop)
+    {
         playlist_t *p_playlist = pl_Get(vd);
         /* Modify playlist as well because the vout might have to be
          * restarted */
@@ -1198,3 +1564,24 @@ static int DesktopCallback(vlc_object_t *object, char const *psz_cmd,
     }
     return VLC_SUCCESS;
 }
+
+
+/*****************************************************************************
+ * FXCallback: Shader-effects variable callback
+ *****************************************************************************/
+static int FXCallback(vlc_object_t *object, char const *psz_cmd,
+                            vlc_value_t oldval, vlc_value_t newval,
+                            void *p_data)
+{
+    vout_display_t *vd = (vout_display_t *)object;
+    vout_display_sys_t *sys = vd->sys;
+    VLC_UNUSED(psz_cmd);
+    VLC_UNUSED(oldval);
+    VLC_UNUSED(p_data);
+
+    vlc_mutex_lock(&sys->lock);
+    sys->shaderFXOption = newval.psz_string;
+    vlc_mutex_unlock(&sys->lock);
+
+    return VLC_SUCCESS;
+}
diff --git a/src/video_output/vout_wrapper.c b/src/video_output/vout_wrapper.c
index 3c2c3f4..cc11fac 100644
--- a/src/video_output/vout_wrapper.c
+++ b/src/video_output/vout_wrapper.c
@@ -84,6 +84,46 @@ int vout_OpenWrapper(vout_thread_t *vout,
     var_AddCallback(vout, "direct3d-desktop", Forward, NULL);
     var_Create(vout, "video-wallpaper", VLC_VAR_BOOL|VLC_VAR_DOINHERIT);
     var_AddCallback(vout, "video-wallpaper", Forward, NULL);
+
+
+    if( var_Type( vout, "shader-effects" ) == 0 )
+    {
+        var_Create(vout, "shader-effects", VLC_VAR_STRING | VLC_VAR_HASCHOICE | VLC_VAR_DOINHERIT);
+        var_AddCallback(vout, "shader-effects", Forward, NULL);
+
+        vlc_value_t val,text;
+        val.psz_string = _("grayscale"); 
+        text.psz_string = _("Grayscale");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+    
+        val.psz_string = _("invert"); 
+        text.psz_string = _("Invert Colors");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+
+        val.psz_string = _("widencolorspace"); 
+        text.psz_string = _("16-235 -> 0-255");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+    
+        val.psz_string = _("detect15"); 
+        text.psz_string = _("Red Detect 0-15 & 236-255");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+
+        val.psz_string = _("convert601to709"); 
+        text.psz_string = _("bt601 -> bt709");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+    
+        val.psz_string = _("gammacorrection18"); 
+        text.psz_string = _("Gamma Correction (1.8)");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+    
+        val.psz_string = _("gammacorrection22"); 
+        text.psz_string = _("Gamma Correction (2.2)");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+    
+        val.psz_string = _("gammacorrectionbt709"); 
+        text.psz_string = _("Gamma Correction (BT.709)");
+        var_Change( vout, "shader-effects", VLC_VAR_ADDCHOICE, &val, &text );
+    }
 #endif
 
     /* */
@@ -102,6 +142,8 @@ void vout_CloseWrapper(vout_thread_t *vout, vout_display_state_t *state)
 #ifdef WIN32
     var_DelCallback(vout, "direct3d-desktop", Forward, NULL);
     var_DelCallback(vout, "video-wallpaper", Forward, NULL);
+    var_DelCallback(vout, "shader-effects", Forward, NULL);
+	var_Destroy( vout, "shader-effects" );
 #endif
     sys->decoder_pool = NULL; /* FIXME remove */
 
-- 
1.7.0.2.msysgit.0

